/* globals pref */
pref("ldap_2.servers.extension.description", "extension");
pref("ldap_2.servers.extension.filename", "ldap1.mab");
pref("ldap_2.servers.extension.uri", "ldap://test.invalid:389/o=invalid??sub");
