/* Any copyright is dedicated to the Public Domain.
   http://creativecommons.org/publicdomain/zero/1.0/ */

/* import-globals-from head_searchconfig.js */

"use strict";

Services.prefs.setIntPref("browser.search.config.test.section", 2);
