// Preferences file used by the raptor harness exclusively on android
/* globals user_pref */

// disk cache smart size is enabled in shipped apps
user_pref("browser.cache.disk.smart_size.enabled", true);
